#!/usr/bin/perl

package Cognition::Export::Location;
use Cognition::Export;
use Cognition::Export::Address;
use Cognition::Export::Point;
use HTML::Entities qw(encode_entities_numeric);;

sub is
{
	my $s = shift;
	return Cognition::Export::Address::is($s)||Cognition::Export::Point::is($s);
}

sub outputs
{
	return [
		{
			code  => 'KML',
			name  => 'Keyhole Markup Language',
			type  => 'application/vnd.google-earth.kml+xml',
			multi => 1
		}
	];
}

sub multiKML
{
	my $triples = shift;
	my $rv;

	foreach my $s (keys %$triples)
	{
		$rv .= _outKML($triples, $s)
			if (defined $triples->{$s} && is($triples->{$s}));
	}

	return  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n"
			. "<kml xmlns=\"http://earth.google.com/kml/2.0\">\r\n"
			. $rv
			. "</kml>\r\n"
			. "<!-- KML output by Cognition. -->\r\n";
}

sub outKML
{
	my $triples = shift;
	my $sURL    = shift;
	
	my $rv = _outKML($triples, $sURL);

	return  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n"
			. "<kml xmlns=\"http://earth.google.com/kml/2.0\">\r\n"
			. $rv
			. "</kml>\r\n"
			. "<!-- KML output by Cognition. -->\r\n";
}

sub _outKML
{
	my $triples = shift;
	my $sURL    = shift;
	my $sObj    = $triples->{$sURL};
	my $rv;
	my $adr;
	
	$rv  = "\t<Placemark>\r\n";
	
	# Co-ordinates
	if (Cognition::Export::Point::is($sObj))
	{
		my $coords = Cognition::Export::Point::outText3($triples, $sURL);
		$rv .= "\t\t<Point><coordinates>$coords</coordinates></Point>\r\n";
	}
	else
	{
		my $geo = firstOf($sObj, 'urn:ietf:rfc:2426#geo',
			'http://www.w3.org/2006/vcard/ns#geo',
			'http://www.w3.org/2006/03/hcard#geo',
			'http://www.w3.org/2001/vcard-rdf/3.0#geo');	
		if ($geo)
		{
			my $coords = Cognition::Export::Point::outText3($triples, $geo);
			$rv .= "\t\t<Point><coordinates>$coords</coordinates></Point>\r\n";
		}
	}

	# Address
	if (Cognition::Export::Address::is($sObj))
	{
		$adr = Cognition::Export::Address::outText($triples, $sURL);
		$rv .= "\t\t<address>".encode_entities_numeric($adr)."</address>\r\n";
	}
	
	# Name
	my $title = firstOf($sObj,
		'http://purl.org/dc/terms/title',
		'http://purl.org/dc/elements/1.1/title') || $adr;
	$rv .= "\t\t<name>".encode_entities_numeric($title)."</name>\r\n"
		if ($title);

	# Description
	my $desc = firstOf($sObj,
		'http://purl.org/dc/terms/description',
		'http://purl.org/dc/elements/1.1/description',
		'http://purl.org/dc/terms/abstract') || $title;
	$rv .= "\t\t<description>".encode_entities_numeric($desc)."</description>\r\n"
		if ($desc);
	
	$rv .= "\t</Placemark>\r\n";
	
	return $rv;
}

1;
