#!/usr/bin/perl

######################################################################
package Cognition::HTMLParser::Utils;
######################################################################

use XML::LibXML qw(:all);
use strict;
use CGI::Util;

BEGIN
{
	our $VERSION   = 0.1;	
	our @outline   = ('outline:', 0, 0, 0, 0, 0, 0);
	our $lastLevel = 0;
}


sub searchClass
{
	my $target = shift;
	my $dom    = shift;
	
	my @matches = ();
	foreach my $node ($dom->getElementsByTagName('*'))
	{
		my $classList = $node->getAttribute('class');
		next unless (length $classList);
		
		if ($classList =~ / (^|\s) $target (\s|$) /x)
		{
			push @matches, $node;
		}
	}
	
	return @matches;
	
} #/sub searchClass


sub searchRel
{
	my $target = shift;
	my $dom    = shift;
	
	$target =~ s/[\:\.]/\[\:\.\]/;
	
	my @matches = ();
	for my $node ($dom->getElementsByTagName('*'))
	{
		my $classList = $node->getAttribute('rel');
		next unless (length $classList);
		
		if ($classList =~ / (^|\s) $target (\s|$) /ix)
		{
			push @matches, $node;
		}
	}
	
	return @matches;
	
} #/sub searchRel


sub searchID
{
	my $target = shift;
	my $dom    = shift;
	
	$target =~ s/^\#//;
	
	my @matches = ();
	for my $node ($dom->getElementsByTagName('*'))
	{
		my $id   = $node->getAttribute('id') || next;
		
		if ($id eq $target)
		{
			return $node;
		}
	}
	
} #/sub searchID


sub outline_point
{
	my $level = shift;
	$level =~ s/[^0-9]//g;
	
	if ($Cognition::HTMLParser::Utils::lastLevel < $level)
	{
		while ($Cognition::HTMLParser::Utils::lastLevel < $level)
		{
			$Cognition::HTMLParser::Utils::lastLevel++;
			$Cognition::HTMLParser::Utils::outline[$Cognition::HTMLParser::Utils::lastLevel] = 1;
		}
	}
	
	elsif ($Cognition::HTMLParser::Utils::lastLevel > $level)
	{
		while ($Cognition::HTMLParser::Utils::lastLevel > $level)
		{
			$Cognition::HTMLParser::Utils::outline[$Cognition::HTMLParser::Utils::lastLevel] = 1;
			$Cognition::HTMLParser::Utils::lastLevel--;
		}
		$Cognition::HTMLParser::Utils::outline[$level]++;
	}
	
	elsif ($Cognition::HTMLParser::Utils::lastLevel == $level)
	{
		$Cognition::HTMLParser::Utils::outline[$level]++;
	}
		
	my @rv = ();
	for (my $i=1; $i<=$level; $i++)
	{
		push @rv, $Cognition::HTMLParser::Utils::outline[$i];
	}
	
	return join('.', @rv);
}


sub STRINGIFY
{
	my $domNode   = shift || return;
	my $withClass = shift || undef;
	my $inClass   = shift || 0;
	my $rv = '';
	
	if (defined $withClass && !$inClass && $domNode->nodeType == XML_ELEMENT_NODE)
	{
		my $c = $domNode->getAttribute('class');
		$inClass = 1 if ($c =~ / (^|\s) $withClass (\s|$) /x);
	}
	
	if ($inClass || !defined $withClass)
	{
		if ($domNode->nodeType == XML_TEXT_NODE)
		{
			$rv = $domNode->getData;
		}
		elsif ($domNode->nodeType == XML_ELEMENT_NODE && $domNode->localname eq 'img')
		{
			$rv = $domNode->getAttribute('alt');
		}
		elsif ($domNode->nodeType == XML_ELEMENT_NODE)
		{
			my @parts;
			foreach my $child ($domNode->getChildNodes)
			{
				push @parts, STRINGIFY($child, $withClass, $inClass);
			}
			$rv = join(' ', @parts);
		}
	}
	
	$rv =~ s/(^\s+|\s+$)//g;
	return $rv;
}


sub xmlns
{
	my $ns  = shift;
	my $dom = shift;
	
	while ($dom->nodeType==XML_ELEMENT_NODE)
	{
		if (length $dom->getAttribute("xmlns:$ns"))
		{
			return $dom->getAttribute("xmlns:$ns");
		}
		$dom = $dom->getParentNode;
	}
	
	if ($dom->nodeType==XML_DOCUMENT_NODE)
	{
		if (length $dom->getDocumentElement->getAttribute("xmlns:$ns"))
		{
			return $dom->getDocumentElement->getAttribute("xmlns:$ns");
		}
	}
	
	return undef;
}

sub PrepareXPointers
# We save the node paths into special @_xpath attributes. Why? Because we often
# need to move DOM nodes about (e.g. uF include pattern) but still want access
# to the original node paths. Sometimes we're just dealing with document
# fragments, which have no meaningful node paths. This seems like an inefficient
# way of doing it, but I can't find a better solution.
{
	my $dom = shift;
	foreach my $tag ($dom->getElementsByTagName('*'))
		{ $tag->setAttribute('_xpath', $tag->nodePath); }
}

sub XPointer
{
	my $dom  = shift;
	my $page = shift || '';
	
	my $xpath = $dom->getAttribute('_xpath');
	if (!length $xpath)
		{ $xpath = $dom->nodePath; }
	
	my $escapedXPath = CGI::Util::escape($xpath);
	$escapedXPath =~ s|\%2f|/|ig;
	$escapedXPath =~ s|\%3f|?|ig;
	$escapedXPath =~ s|\%5b|[|ig;
	$escapedXPath =~ s|\%5d|]|ig;
	return "$page#xpointer($escapedXPath)";
}


1; #/package Cognition::HTMLParser::Utils




