#!/usr/bin/perl

######################################################################
package Cognition::uF::adr;
######################################################################

use XML::LibXML qw(:all);
use strict;
use CGI::Util;
use Digest::SHA1 qw(sha1_hex);

BEGIN
{
	our $VERSION = 0.1;
	our @ISA = qw(Cognition::uF);
}


sub parse_all
{
	my $page   = shift;
	my $within = shift || $page->{DOM};
	my @nodes = Cognition::HTMLParser::Utils::searchClass('adr', $within);
	my @rv;
	
	foreach my $adr (@nodes)
	{
		my $adr_parsed = parse($page, $adr);
		push @rv, $adr_parsed;
	}
	
	return @rv;
} #/sub parse_all


sub uri
{
	my $this = shift;
	my $all  = shift;
	my @rv;
	
	if (length $this->{_id})
	{
		push @rv, $this->{_page}->abs_url.'#'.$this->{_id};
	}
	push @rv, Cognition::HTMLParser::Utils::XPointer($this->{_dom}, $this->{_page}->abs_url);
	
	return $rv[0] unless (defined $all);
	while ($all) { shift @rv; $all--; }
	return @rv;
}


sub metadata_ns
{
	my $this = shift;
	return { 'VCARD' => { nsurl=>'urn:ietf:rfc:2426#',title=>'vCard' } };
} #/sub namespace


sub rdf_subject
{
	my $this    = shift;
	my $ns      = 'urn:ietf:rfc:2426#';

	# Create RDF subject
	my $subject = Cognition::RDFModel::Subject->new($this->uri);
	$subject->addObject($this);
	$subject->setDom($this->{_dom});

	$subject->addVal('http://www.w3.org/1999/02/22-rdf-syntax-ns#type',
		Cognition::RDFModel::Value->new($ns.'adr', 1));

	# Patterns for simple data.
	my @_MULTIS   = qw(post-office-box extended-address street-address locality
		region postal-code country-name type);
	my @_OBJECTS  = qw(geo);
	
	# Properties which exist as multiple values.
	foreach my $property (@_MULTIS)
	{
		foreach my $V (@{ $this->{$property} })
		{
			next unless (length $V);
			my $val = Cognition::RDFModel::Value->new($V);
			$subject->addVal($ns.$property, $val);
		}
	}
	
	# Embedded objects which may be represented as resources
	foreach my $property (@_OBJECTS)
	{
		foreach my $V (@{ $this->{$property} })
		{
			my $val;
			if (ref $V)
				{ $val = Cognition::RDFModel::Value->new($V->uri, 1); }
			else
				{ $val = Cognition::RDFModel::Value->new($V); }
			$subject->addVal($ns.$property, $val);
		}
	}

	return $subject;
	
} #/sub rdf_subject


sub parse
{
	my $page = shift;
	my $rv = { '_dom'=>shift };
	my $adr_node = $rv->{'_dom'}->cloneNode(1);

	my $id = $adr_node->getAttribute('id');
	$rv->{_id} = $id if (length $id);
	$rv->{_page} = $page;
	
	# Expand microformat patterns.
	my $incl_iterations = 0;
	my $replacements = 1;
	while (($incl_iterations < Cognition::uF::MAX_INCLUDE_ITERATIONS) && $replacements)
	{
		$replacements = Cognition::uF::ufp_include($adr_node, $page) 
		              + Cognition::uF::ufp_include2($adr_node, $page);
		$incl_iterations++;
	}
	Cognition::uF::ufp_abbr($adr_node, $page);
	Cognition::uF::ufp_table($adr_node, $page);
	
	# Geo - an extension to adr spec
	my @geo = Cognition::uF::geo::parse_all($page, $adr_node);
	$rv->{geo} = \@geo unless (!@geo);
	
	# Now that we have reached here, no other composite microformats have
	# any business being nested within this root element. So in the interest
	# of Microformat opacity, let's destroy them.
	Cognition::uF::destroyer($adr_node);

	# Simple attributes - possible duplicates
	my @simples = qw(post-office-box extended-address street-address locality
		region postal-code country-name type);	
	foreach my $simple (@simples)
	{
		my @value_nodes = Cognition::HTMLParser::Utils::searchClass($simple, $adr_node);
		foreach my $value_node (@value_nodes)
		{
			my $value = Cognition::HTMLParser::Utils::STRINGIFY($value_node);
			push @{$rv->{$simple}}, $value;
		}
	}

	bless $rv;
	return $rv;
	
} #/sub parse


1; #/package Cognition::uF::adr




